@@ -44,7 +44,7 @@ import org.eclipse.aether.transform.FileTransformerManager;
  * supposed to be immutable and hence can safely be shared across an entire application and any concurrent threads
  * reading it. Components that wish to tweak some aspects of an existing session should use the copy constructor of
  * {@link DefaultRepositorySystemSession} and its mutators to derive a custom session.
- * 
+ *
  * @noimplement This interface is not intended to be implemented by clients.
  * @noextend This interface is not intended to be extended by clients.
  */
@@ -265,7 +265,9 @@ public interface RepositorySystemSession
      * Get the file transformer manager
      * 
      * @return the manager, never {@code null}
+     * @deprecated Without any direct replacement for now. This API is OOM-prone, and also lacks a lot of context about
+     * transforming.
      */
+    @Deprecated
     FileTransformerManager getFileTransformerManager();
-
 }
