@@ -1,21 +1,32 @@
-/*******************************************************************************
- * Copyright (c) 2010, 2014 Sonatype, Inc.
- * All rights reserved. This program and the accompanying materials
- * are made available under the terms of the Eclipse Public License v1.0
- * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
- *
- * Contributors:
- *    Sonatype, Inc. - initial API and implementation
- *******************************************************************************/
 package org.eclipse.aether.internal.impl;
 
+/*
+ * Licensed to the Apache Software Foundation (ASF) under one
+ * or more contributor license agreements.  See the NOTICE file
+ * distributed with this work for additional information
+ * regarding copyright ownership.  The ASF licenses this file
+ * to you under the Apache License, Version 2.0 (the
+ * "License"); you may not use this file except in compliance
+ * with the License.  You may obtain a copy of the License at
+ *
+ *  http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing,
+ * software distributed under the License is distributed on an
+ * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
+ * KIND, either express or implied.  See the License for the
+ * specific language governing permissions and limitations
+ * under the License.
+ */
+
+import javax.inject.Inject;
+import javax.inject.Named;
+import javax.inject.Singleton;
+
 import java.util.ArrayList;
 import java.util.Collection;
 import java.util.List;
-
-import javax.inject.Inject;
-import javax.inject.Named;
+import java.util.concurrent.atomic.AtomicBoolean;
 
 import org.eclipse.aether.RepositorySystem;
 import org.eclipse.aether.RepositorySystemSession;
@@ -38,7 +49,7 @@ import org.eclipse.aether.impl.Installer;
 import org.eclipse.aether.impl.LocalRepositoryProvider;
 import org.eclipse.aether.impl.MetadataResolver;
 import org.eclipse.aether.impl.RemoteRepositoryManager;
-import org.eclipse.aether.impl.SyncContextFactory;
+import org.eclipse.aether.impl.RepositorySystemLifecycle;
 import org.eclipse.aether.impl.VersionRangeResolver;
 import org.eclipse.aether.impl.VersionResolver;
 import org.eclipse.aether.installation.InstallRequest;
@@ -69,20 +80,21 @@ import org.eclipse.aether.resolution.VersionResolutionException;
 import org.eclipse.aether.resolution.VersionResult;
 import org.eclipse.aether.spi.locator.Service;
 import org.eclipse.aether.spi.locator.ServiceLocator;
-import org.eclipse.aether.spi.log.Logger;
-import org.eclipse.aether.spi.log.LoggerFactory;
-import org.eclipse.aether.spi.log.NullLoggerFactory;
+import org.eclipse.aether.spi.synccontext.SyncContextFactory;
 import org.eclipse.aether.util.graph.visitor.FilteringDependencyVisitor;
 import org.eclipse.aether.util.graph.visitor.TreeDependencyVisitor;
 
+import static java.util.Objects.requireNonNull;
+
 /**
+ *
  */
+@Singleton
 @Named
 public class DefaultRepositorySystem
-    implements RepositorySystem, Service
+        implements RepositorySystem, Service
 {
-
-    private Logger logger = NullLoggerFactory.LOGGER;
+    private final AtomicBoolean shutdown;
 
     private VersionResolver versionResolver;
 
@@ -106,19 +118,25 @@ public class DefaultRepositorySystem
 
     private RemoteRepositoryManager remoteRepositoryManager;
 
+    private RepositorySystemLifecycle repositorySystemLifecycle;
+
     public DefaultRepositorySystem()
     {
         // enables default constructor
+        this.shutdown = new AtomicBoolean( false );
     }
 
+    @SuppressWarnings( "checkstyle:parameternumber" )
     @Inject
     DefaultRepositorySystem( VersionResolver versionResolver, VersionRangeResolver versionRangeResolver,
                              ArtifactResolver artifactResolver, MetadataResolver metadataResolver,
                              ArtifactDescriptorReader artifactDescriptorReader,
                              DependencyCollector dependencyCollector, Installer installer, Deployer deployer,
                              LocalRepositoryProvider localRepositoryProvider, SyncContextFactory syncContextFactory,
-                             RemoteRepositoryManager remoteRepositoryManager, LoggerFactory loggerFactory )
+                             RemoteRepositoryManager remoteRepositoryManager,
+                             RepositorySystemLifecycle repositorySystemLifecycle )
     {
+        this.shutdown = new AtomicBoolean( false );
         setVersionResolver( versionResolver );
         setVersionRangeResolver( versionRangeResolver );
         setArtifactResolver( artifactResolver );
@@ -130,12 +148,12 @@ public class DefaultRepositorySystem
         setLocalRepositoryProvider( localRepositoryProvider );
         setSyncContextFactory( syncContextFactory );
         setRemoteRepositoryManager( remoteRepositoryManager );
-        setLoggerFactory( loggerFactory );
+        setRepositorySystemLifecycle( repositorySystemLifecycle );
     }
 
+    @Override
     public void initService( ServiceLocator locator )
     {
-        setLoggerFactory( locator.getService( LoggerFactory.class ) );
         setVersionResolver( locator.getService( VersionResolver.class ) );
         setVersionRangeResolver( locator.getService( VersionRangeResolver.class ) );
         setArtifactResolver( locator.getService( ArtifactResolver.class ) );
@@ -147,179 +165,174 @@ public class DefaultRepositorySystem
         setLocalRepositoryProvider( locator.getService( LocalRepositoryProvider.class ) );
         setRemoteRepositoryManager( locator.getService( RemoteRepositoryManager.class ) );
         setSyncContextFactory( locator.getService( SyncContextFactory.class ) );
+        setRepositorySystemLifecycle( locator.getService( RepositorySystemLifecycle.class ) );
     }
 
-    public DefaultRepositorySystem setLoggerFactory( LoggerFactory loggerFactory )
+    /**
+     * @deprecated not used any more since MRESOLVER-36 move to slf4j, added back in MRESOLVER-64 for compatibility
+     */
+    @Deprecated
+    public DefaultRepositorySystem setLoggerFactory( org.eclipse.aether.spi.log.LoggerFactory loggerFactory )
     {
-        this.logger = NullLoggerFactory.getSafeLogger( loggerFactory, getClass() );
+        // this.logger = NullLoggerFactory.getSafeLogger( loggerFactory, getClass() );
         return this;
     }
 
     public DefaultRepositorySystem setVersionResolver( VersionResolver versionResolver )
     {
-        if ( versionResolver == null )
-        {
-            throw new IllegalArgumentException( "version resolver has not been specified" );
-        }
-        this.versionResolver = versionResolver;
+        this.versionResolver = requireNonNull( versionResolver, "version resolver cannot be null" );
         return this;
     }
 
     public DefaultRepositorySystem setVersionRangeResolver( VersionRangeResolver versionRangeResolver )
     {
-        if ( versionRangeResolver == null )
-        {
-            throw new IllegalArgumentException( "version range resolver has not been specified" );
-        }
-        this.versionRangeResolver = versionRangeResolver;
+        this.versionRangeResolver = requireNonNull(
+                versionRangeResolver, "version range resolver cannot be null" );
         return this;
     }
 
     public DefaultRepositorySystem setArtifactResolver( ArtifactResolver artifactResolver )
     {
-        if ( artifactResolver == null )
-        {
-            throw new IllegalArgumentException( "artifact resolver has not been specified" );
-        }
-        this.artifactResolver = artifactResolver;
+        this.artifactResolver = requireNonNull( artifactResolver, "artifact resolver cannot be null" );
         return this;
     }
 
     public DefaultRepositorySystem setMetadataResolver( MetadataResolver metadataResolver )
     {
-        if ( metadataResolver == null )
-        {
-            throw new IllegalArgumentException( "metadata resolver has not been specified" );
-        }
-        this.metadataResolver = metadataResolver;
+        this.metadataResolver = requireNonNull( metadataResolver, "metadata resolver cannot be null" );
         return this;
     }
 
     public DefaultRepositorySystem setArtifactDescriptorReader( ArtifactDescriptorReader artifactDescriptorReader )
     {
-        if ( artifactDescriptorReader == null )
-        {
-            throw new IllegalArgumentException( "artifact descriptor reader has not been specified" );
-        }
-        this.artifactDescriptorReader = artifactDescriptorReader;
+        this.artifactDescriptorReader = requireNonNull(
+                artifactDescriptorReader, "artifact descriptor reader cannot be null" );
         return this;
     }
 
     public DefaultRepositorySystem setDependencyCollector( DependencyCollector dependencyCollector )
     {
-        if ( dependencyCollector == null )
-        {
-            throw new IllegalArgumentException( "dependency collector has not been specified" );
-        }
-        this.dependencyCollector = dependencyCollector;
+        this.dependencyCollector = requireNonNull( dependencyCollector, "dependency collector cannot be null" );
         return this;
     }
 
     public DefaultRepositorySystem setInstaller( Installer installer )
     {
-        if ( installer == null )
-        {
-            throw new IllegalArgumentException( "installer has not been specified" );
-        }
-        this.installer = installer;
+        this.installer = requireNonNull( installer, "installer cannot be null" );
         return this;
     }
 
     public DefaultRepositorySystem setDeployer( Deployer deployer )
     {
-        if ( deployer == null )
-        {
-            throw new IllegalArgumentException( "deployer has not been specified" );
-        }
-        this.deployer = deployer;
+        this.deployer = requireNonNull( deployer, "deployer cannot be null" );
         return this;
     }
 
     public DefaultRepositorySystem setLocalRepositoryProvider( LocalRepositoryProvider localRepositoryProvider )
     {
-        if ( localRepositoryProvider == null )
-        {
-            throw new IllegalArgumentException( "local repository provider has not been specified" );
-        }
-        this.localRepositoryProvider = localRepositoryProvider;
+        this.localRepositoryProvider = requireNonNull(
+                localRepositoryProvider, "local repository provider cannot be null" );
         return this;
     }
 
     public DefaultRepositorySystem setSyncContextFactory( SyncContextFactory syncContextFactory )
     {
-        if ( syncContextFactory == null )
-        {
-            throw new IllegalArgumentException( "sync context factory has not been specified" );
-        }
-        this.syncContextFactory = syncContextFactory;
+        this.syncContextFactory = requireNonNull( syncContextFactory, "sync context factory cannot be null" );
         return this;
     }
 
     public DefaultRepositorySystem setRemoteRepositoryManager( RemoteRepositoryManager remoteRepositoryManager )
     {
-        if ( remoteRepositoryManager == null )
-        {
-            throw new IllegalArgumentException( "remote repository manager has not been specified" );
-        }
-        this.remoteRepositoryManager = remoteRepositoryManager;
+        this.remoteRepositoryManager = requireNonNull(
+                remoteRepositoryManager, "remote repository provider cannot be null" );
         return this;
     }
 
+    public DefaultRepositorySystem setRepositorySystemLifecycle( RepositorySystemLifecycle repositorySystemLifecycle )
+    {
+        this.repositorySystemLifecycle = requireNonNull(
+                repositorySystemLifecycle, "repository system lifecycle cannot be null" );
+        return this;
+    }
+
+    @Override
     public VersionResult resolveVersion( RepositorySystemSession session, VersionRequest request )
-        throws VersionResolutionException
+            throws VersionResolutionException
     {
         validateSession( session );
+        requireNonNull( request, "request cannot be null" );
+
         return versionResolver.resolveVersion( session, request );
     }
 
+    @Override
     public VersionRangeResult resolveVersionRange( RepositorySystemSession session, VersionRangeRequest request )
-        throws VersionRangeResolutionException
+            throws VersionRangeResolutionException
     {
         validateSession( session );
+        requireNonNull( request, "request cannot be null" );
+
         return versionRangeResolver.resolveVersionRange( session, request );
     }
 
+    @Override
     public ArtifactDescriptorResult readArtifactDescriptor( RepositorySystemSession session,
                                                             ArtifactDescriptorRequest request )
-        throws ArtifactDescriptorException
+            throws ArtifactDescriptorException
     {
         validateSession( session );
+        requireNonNull( request, "request cannot be null" );
+
         return artifactDescriptorReader.readArtifactDescriptor( session, request );
     }
 
+    @Override
     public ArtifactResult resolveArtifact( RepositorySystemSession session, ArtifactRequest request )
-        throws ArtifactResolutionException
+            throws ArtifactResolutionException
     {
         validateSession( session );
+        requireNonNull( session, "session cannot be null" );
+
         return artifactResolver.resolveArtifact( session, request );
     }
 
+    @Override
     public List<ArtifactResult> resolveArtifacts( RepositorySystemSession session,
                                                   Collection<? extends ArtifactRequest> requests )
-        throws ArtifactResolutionException
+            throws ArtifactResolutionException
     {
         validateSession( session );
+        requireNonNull( requests, "requests cannot be null" );
+
         return artifactResolver.resolveArtifacts( session, requests );
     }
 
+    @Override
     public List<MetadataResult> resolveMetadata( RepositorySystemSession session,
                                                  Collection<? extends MetadataRequest> requests )
     {
         validateSession( session );
+        requireNonNull( requests, "requests cannot be null" );
+
         return metadataResolver.resolveMetadata( session, requests );
     }
 
+    @Override
     public CollectResult collectDependencies( RepositorySystemSession session, CollectRequest request )
-        throws DependencyCollectionException
+            throws DependencyCollectionException
     {
         validateSession( session );
+        requireNonNull( request, "request cannot be null" );
+
         return dependencyCollector.collectDependencies( session, request );
     }
 
+    @Override
     public DependencyResult resolveDependencies( RepositorySystemSession session, DependencyRequest request )
-        throws DependencyResolutionException
+            throws DependencyResolutionException
     {
         validateSession( session );
+        requireNonNull( request, "request cannot be null" );
 
         RequestTrace trace = RequestTrace.newChild( request.getTrace(), request );
 
@@ -351,14 +364,19 @@ public class DefaultRepositorySystem
         }
         else
         {
-            throw new IllegalArgumentException( "dependency node or collect request missing" );
+            throw new NullPointerException( "dependency node and collect request cannot be null" );
         }
 
         ArtifactRequestBuilder builder = new ArtifactRequestBuilder( trace );
         DependencyFilter filter = request.getFilter();
         DependencyVisitor visitor = ( filter != null ) ? new FilteringDependencyVisitor( builder, filter ) : builder;
         visitor = new TreeDependencyVisitor( visitor );
-        result.getRoot().accept( visitor );
+
+        if ( result.getRoot() != null )
+        {
+            result.getRoot().accept( visitor );
+        }
+
         List<ArtifactRequest> requests = builder.getRequests();
 
         List<ArtifactResult> results;
@@ -399,23 +417,33 @@ public class DefaultRepositorySystem
         }
     }
 
+    @Override
     public InstallResult install( RepositorySystemSession session, InstallRequest request )
-        throws InstallationException
+            throws InstallationException
     {
         validateSession( session );
+        requireNonNull( request, "request cannot be null" );
+
         return installer.install( session, request );
     }
 
+    @Override
     public DeployResult deploy( RepositorySystemSession session, DeployRequest request )
-        throws DeploymentException
+            throws DeploymentException
     {
         validateSession( session );
+        requireNonNull( request, "request cannot be null" );
+
         return deployer.deploy( session, request );
     }
 
+    @Override
     public LocalRepositoryManager newLocalRepositoryManager( RepositorySystemSession session,
                                                              LocalRepository localRepository )
     {
+        requireNonNull( session, "session cannot be null" );
+        requireNonNull( localRepository, "localRepository cannot be null" );
+
         try
         {
             return localRepositoryProvider.newLocalRepositoryManager( session, localRepository );
@@ -426,25 +454,32 @@ public class DefaultRepositorySystem
         }
     }
 
+    @Override
     public SyncContext newSyncContext( RepositorySystemSession session, boolean shared )
     {
         validateSession( session );
         return syncContextFactory.newInstance( session, shared );
     }
 
+    @Override
     public List<RemoteRepository> newResolutionRepositories( RepositorySystemSession session,
                                                              List<RemoteRepository> repositories )
     {
         validateSession( session );
+        validateRepositories( repositories );
+
         repositories =
-            remoteRepositoryManager.aggregateRepositories( session, new ArrayList<RemoteRepository>(), repositories,
-                                                           true );
+                remoteRepositoryManager.aggregateRepositories( session, new ArrayList<RemoteRepository>(), repositories,
+                        true );
         return repositories;
     }
 
+    @Override
     public RemoteRepository newDeploymentRepository( RepositorySystemSession session, RemoteRepository repository )
     {
         validateSession( session );
+        requireNonNull( repository, "repository cannot be null" );
+
         RemoteRepository.Builder builder = new RemoteRepository.Builder( repository );
         Authentication auth = session.getAuthenticationSelector().getAuthentication( repository );
         builder.setAuthentication( auth );
@@ -453,53 +488,50 @@ public class DefaultRepositorySystem
         return builder.build();
     }
 
+    @Override
+    public void addOnSystemEndedHandler( Runnable handler )
+    {
+        repositorySystemLifecycle.addOnSystemEndedHandler( handler );
+    }
+
+    @Override
+    public void shutdown()
+    {
+        if ( shutdown.compareAndSet( false, true ) )
+        {
+            repositorySystemLifecycle.systemEnded();
+        }
+    }
+
     private void validateSession( RepositorySystemSession session )
     {
-        if ( session == null )
+        requireNonNull( session, "repository system session cannot be null" );
+        invalidSession( session.getLocalRepositoryManager(), "local repository manager" );
+        invalidSession( session.getSystemProperties(), "system properties" );
+        invalidSession( session.getUserProperties(), "user properties" );
+        invalidSession( session.getConfigProperties(), "config properties" );
+        invalidSession( session.getMirrorSelector(), "mirror selector" );
+        invalidSession( session.getProxySelector(), "proxy selector" );
+        invalidSession( session.getAuthenticationSelector(), "authentication selector" );
+        invalidSession( session.getArtifactTypeRegistry(), "artifact type registry" );
+        invalidSession( session.getData(), "data" );
+        if ( shutdown.get() )
         {
-            throw new IllegalArgumentException( "Invalid repository system session: the session may not be null." );
-        }
-        if ( session.getLocalRepositoryManager() == null )
-        {
-            invalidSession( "LocalRepositoryManager" );
-        }
-        if ( session.getSystemProperties() == null )
-        {
-            invalidSession( "SystemProperties" );
-        }
-        if ( session.getUserProperties() == null )
-        {
-            invalidSession( "UserProperties" );
-        }
-        if ( session.getConfigProperties() == null )
-        {
-            invalidSession( "ConfigProperties" );
-        }
-        if ( session.getMirrorSelector() == null )
-        {
-            invalidSession( "MirrorSelector" );
-        }
-        if ( session.getProxySelector() == null )
-        {
-            invalidSession( "ProxySelector" );
-        }
-        if ( session.getAuthenticationSelector() == null )
-        {
-            invalidSession( "AuthenticationSelector" );
-        }
-        if ( session.getArtifactTypeRegistry() == null )
-        {
-            invalidSession( "ArtifactTypeRegistry" );
-        }
-        if ( session.getData() == null )
-        {
-            invalidSession( "Data" );
+            throw new IllegalStateException( "repository system is already shut down" );
         }
     }
 
-    private void invalidSession( String name )
+    private void validateRepositories( List<RemoteRepository> repositories )
     {
-        throw new IllegalArgumentException( "Invalid repository system session: " + name + " is not set." );
+        requireNonNull( repositories, "repositories cannot be null" );
+        for ( RemoteRepository repository : repositories )
+        {
+            requireNonNull( repository, "repository cannot be null" );
+        }
     }
 
+    private void invalidSession( Object obj, String name )
+    {
+        requireNonNull( obj, "repository system session's " + name + " cannot be null" );
+    }
 }
