@@ -108,6 +108,12 @@ public class FilteringParserDelegate extends JsonParserDelegate
         this(p, f, includePath ? Inclusion.INCLUDE_ALL_AND_PATH : Inclusion.ONLY_INCLUDE_ALL, allowMultipleMatches);
     }
 
+    /**
+     * @param p Parser to delegate calls to
+     * @param f Filter to use
+     * @param inclusion Definition of inclusion criteria
+     * @param allowMultipleMatches Whether to allow multiple matches
+     */
     public FilteringParserDelegate(JsonParser p, TokenFilter f,
             TokenFilter.Inclusion inclusion, boolean allowMultipleMatches)
     {
@@ -131,6 +137,8 @@ public class FilteringParserDelegate extends JsonParserDelegate
     /**
      * Accessor for finding number of matches, where specific token and sub-tree
      * starting (if structured type) are passed.
+     *
+     * @return Number of matches
      */
     public int getMatchCount() {
         return _matchCount;
@@ -176,7 +184,7 @@ public class FilteringParserDelegate extends JsonParserDelegate
     public JsonStreamContext getParsingContext() {
         return _filterContext();
     }
-    
+
     // !!! TODO: Verify it works as expected: copied from standard JSON parser impl
     @Override
     public String getCurrentName() throws IOException {
@@ -188,6 +196,17 @@ public class FilteringParserDelegate extends JsonParserDelegate
         return ctxt.getCurrentName();
     }
 
+    // 2.13: IMPORTANT! Must override along with older getCurrentName()
+    @Override
+    public String currentName() throws IOException {
+        JsonStreamContext ctxt = _filterContext();
+        if (_currToken == JsonToken.START_OBJECT || _currToken == JsonToken.START_ARRAY) {
+            JsonStreamContext parent = ctxt.getParent();
+            return (parent == null) ? null : parent.getCurrentName();
+        }
+        return ctxt.getCurrentName();
+    }
+
     /*
     /**********************************************************
     /* Public API, token state overrides
@@ -207,10 +226,9 @@ public class FilteringParserDelegate extends JsonParserDelegate
 
     @Override
     public void overrideCurrentName(String name) {
-        /* 14-Apr-2015, tatu: Not sure whether this can be supported, and if so,
-         *    what to do with it... Delegation won't work for sure, so let's for
-         *    now throw an exception
-         */
+        // 14-Apr-2015, tatu: Not sure whether this can be supported, and if so,
+        //    what to do with it... Delegation won't work for sure, so let's for
+        //    now throw an exception
         throw new UnsupportedOperationException("Can not currently override name during filtering read");
     }
 
@@ -255,21 +273,26 @@ public class FilteringParserDelegate extends JsonParserDelegate
                     _exposedContext = null;
                     if (ctxt.inArray()) {
                         t = delegate.getCurrentToken();
-// Is this guaranteed to work without further checks?
-//                        if (t != JsonToken.START_ARRAY) {
                         _currToken = t;
+                        if (_currToken == JsonToken.END_ARRAY) {
+                            _headContext = _headContext.getParent();
+                            _itemFilter = _headContext.getFilter();
+                        }
                         return t;
                     }
 
+                    // 19-Jul-2021, tatu: [core#700]: following was commented out?!
                     // Almost! Most likely still have the current token;
-                    // with the sole exception of 
-                    /*
-                    t = delegate.getCurrentToken();
+                    // with the sole exception of FIELD_NAME
+                    t = delegate.currentToken();
+                    if (t == JsonToken.END_OBJECT) {
+                        _headContext = _headContext.getParent();
+                        _itemFilter = _headContext.getFilter();
+                    }
                     if (t != JsonToken.FIELD_NAME) {
                         _currToken = t;
                         return t;
                     }
-                    */
                     break;
                 }
                 // If not, traverse down the context chain
@@ -453,12 +476,10 @@ public class FilteringParserDelegate extends JsonParserDelegate
         return _nextToken2();
     }
 
-    /**
-     * Offlined handling for cases where there was no buffered token to
-     * return, and the token read next could not be returned as-is,
-     * at least not yet, but where we have not yet established that
-     * buffering is needed.
-     */
+    // Offlined handling for cases where there was no buffered token to
+    // return, and the token read next could not be returned as-is,
+    // at least not yet, but where we have not yet established that
+    // buffering is needed.
     protected final JsonToken _nextToken2() throws IOException
     {
         main_loop:
@@ -547,13 +568,33 @@ public class FilteringParserDelegate extends JsonParserDelegate
                 continue main_loop;
 
             case ID_END_ARRAY:
+                {
+                    boolean returnEnd = _headContext.isStartHandled();
+                    f = _headContext.getFilter();
+                    if ((f != null) && (f != TokenFilter.INCLUDE_ALL)) {
+                        boolean includeEmpty = f.includeEmptyArray(_headContext.hasCurrentIndex());
+                        f.filterFinishArray();
+                        if (includeEmpty) {
+                            return _nextBuffered(_headContext);
+                        }
+                    }
+                    _headContext = _headContext.getParent();
+                    _itemFilter = _headContext.getFilter();
+                    if (returnEnd) {
+                        return (_currToken = t);
+                    }
+                }
+                continue main_loop;
             case ID_END_OBJECT:
                 {
                     boolean returnEnd = _headContext.isStartHandled();
                     f = _headContext.getFilter();
                     if ((f != null) && (f != TokenFilter.INCLUDE_ALL)) {
-                        f.filterFinishArray();
-                    }
+                        boolean includeEmpty = f.includeEmptyArray(_headContext.hasCurrentName());
+                        f.filterFinishObject();
+                        if (includeEmpty) {
+                            return _nextBuffered(_headContext);
+                        }                    }
                     _headContext = _headContext.getParent();
                     _itemFilter = _headContext.getFilter();
                     if (returnEnd) {
@@ -583,8 +624,13 @@ public class FilteringParserDelegate extends JsonParserDelegate
                     }
                     _itemFilter = f;
                     if (f == TokenFilter.INCLUDE_ALL) {
-                        if (_verifyAllowedMatches() && _inclusion == Inclusion.INCLUDE_ALL_AND_PATH) {
-                            return (_currToken = t);
+                        if (_verifyAllowedMatches()) {
+                            if (_inclusion == Inclusion.INCLUDE_ALL_AND_PATH) {
+                                return (_currToken = t);
+                            }
+                        } else {
+                            delegate.nextToken();
+                            delegate.skipChildren();
                         }
                         continue main_loop;
                     }
@@ -618,9 +664,7 @@ public class FilteringParserDelegate extends JsonParserDelegate
         }
     }
 
-    /**
-     * Method called when a new potentially included context is found.
-     */
+    // Method called when a new potentially included context is found.
     protected final JsonToken _nextTokenWithBuffering(final TokenFilterContext buffRoot)
         throws IOException
     {
@@ -690,13 +734,16 @@ public class FilteringParserDelegate extends JsonParserDelegate
                 continue main_loop;
 
             case ID_END_ARRAY:
-            case ID_END_OBJECT:
                 {
                     // Unlike with other loops, here we know that content was NOT
                     // included (won't get this far otherwise)
                     f = _headContext.getFilter();
                     if ((f != null) && (f != TokenFilter.INCLUDE_ALL)) {
+                        boolean includeEmpty = f.includeEmptyArray(_headContext.hasCurrentIndex());
                         f.filterFinishArray();
+                        if (includeEmpty) {
+                            return _nextBuffered(buffRoot);
+                        }
                     }
                     boolean gotEnd = (_headContext == buffRoot);
                     boolean returnEnd = gotEnd && _headContext.isStartHandled();
@@ -709,6 +756,33 @@ public class FilteringParserDelegate extends JsonParserDelegate
                     }
                 }
                 continue main_loop;
+            case ID_END_OBJECT:
+            {
+                // Unlike with other loops, here we know that content was NOT
+                // included (won't get this far otherwise)
+                f = _headContext.getFilter();
+                if ((f != null) && (f != TokenFilter.INCLUDE_ALL)) {
+                    boolean includeEmpty = f.includeEmptyObject(_headContext.hasCurrentName());
+                    f.filterFinishObject();
+                    if (includeEmpty) {
+                        _headContext._currentName = _headContext._parent == null
+                                ? null
+                                : _headContext._parent._currentName;
+                        _headContext._needToHandleName = false;
+                        return _nextBuffered(buffRoot);
+                    }
+                }
+                boolean gotEnd = (_headContext == buffRoot);
+                boolean returnEnd = gotEnd && _headContext.isStartHandled();
+
+                _headContext = _headContext.getParent();
+                _itemFilter = _headContext.getFilter();
+
+                if (returnEnd) {
+                    return t;
+                }
+            }
+            continue main_loop;
 
             case ID_FIELD_NAME:
                 {
@@ -847,11 +921,44 @@ public class FilteringParserDelegate extends JsonParserDelegate
     /**********************************************************
      */
 
-    @Override public String getText() throws IOException { return delegate.getText();  }
-    @Override public boolean hasTextCharacters() { return delegate.hasTextCharacters(); }
-    @Override public char[] getTextCharacters() throws IOException { return delegate.getTextCharacters(); }
-    @Override public int getTextLength() throws IOException { return delegate.getTextLength(); }
-    @Override public int getTextOffset() throws IOException { return delegate.getTextOffset(); }
+    // 19-Jul-2021, tatu: Cannot quite just delegate these methods due to oddity
+    //   of property name token, which may be buffered.
+
+    @Override public String getText() throws IOException {
+        if (_currToken == JsonToken.FIELD_NAME) {
+            return currentName();
+        }
+        return delegate.getText();
+    }
+
+    @Override public boolean hasTextCharacters() {
+        if (_currToken == JsonToken.FIELD_NAME) {
+            return false;
+        }
+        return delegate.hasTextCharacters();
+    }
+
+    @Override public char[] getTextCharacters() throws IOException {
+        // Not optimal but is correct, unlike delegating (as underlying stream
+        // may point to something else due to buffering)
+        if (_currToken == JsonToken.FIELD_NAME) {
+            return currentName().toCharArray();
+        }
+        return delegate.getTextCharacters();
+    }
+
+    @Override public int getTextLength() throws IOException {
+        if (_currToken == JsonToken.FIELD_NAME) {
+            return currentName().length();
+        }
+        return delegate.getTextLength();
+    }
+    @Override public int getTextOffset() throws IOException {
+        if (_currToken == JsonToken.FIELD_NAME) {
+            return 0;
+        }
+        return delegate.getTextOffset();
+    }
 
     /*
     /**********************************************************
@@ -906,9 +1013,20 @@ public class FilteringParserDelegate extends JsonParserDelegate
     @Override public double getValueAsDouble(double defaultValue) throws IOException { return delegate.getValueAsDouble(defaultValue); }
     @Override public boolean getValueAsBoolean() throws IOException { return delegate.getValueAsBoolean(); }
     @Override public boolean getValueAsBoolean(boolean defaultValue) throws IOException { return delegate.getValueAsBoolean(defaultValue); }
-    @Override public String getValueAsString() throws IOException { return delegate.getValueAsString(); }
-    @Override public String getValueAsString(String defaultValue) throws IOException { return delegate.getValueAsString(defaultValue); }
-    
+
+    @Override public String getValueAsString() throws IOException {
+        if (_currToken == JsonToken.FIELD_NAME) {
+            return currentName();
+        }
+        return delegate.getValueAsString();
+    }
+    @Override public String getValueAsString(String defaultValue) throws IOException {
+        if (_currToken == JsonToken.FIELD_NAME) {
+            return currentName();
+        }
+        return delegate.getValueAsString(defaultValue);
+    }
+
     /*
     /**********************************************************
     /* Public API, access to token values, other
@@ -932,5 +1050,4 @@ public class FilteringParserDelegate extends JsonParserDelegate
         }
         return _headContext;
     }
-  
 }
