@@ -5,7 +5,7 @@
  * you may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
  *
- * http://www.apache.org/licenses/LICENSE-2.0
+ * https://www.apache.org/licenses/LICENSE-2.0
  *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
@@ -15,6 +15,7 @@
  */
 package io.micronaut.context;
 
+import io.micronaut.context.annotation.Primary;
 import io.micronaut.inject.BeanType;
 
 import java.util.Optional;
@@ -22,7 +23,7 @@ import java.util.stream.Stream;
 
 /**
  * <p>Used to qualify which bean to select in the case of multiple possible options.</p>
- * <p>
+ *
  * <p>NOTE: When implementing a custom Qualifier you MUST implement {@link Object#hashCode()} and
  * {@link Object#equals(Object)} so that the qualifier can be used in comparisons and equality checks</p>
  *
@@ -31,6 +32,10 @@ import java.util.stream.Stream;
  * @since 1.0
  */
 public interface Qualifier<T> {
+    /**
+     * The name of the primary annotation class.
+     */
+    String PRIMARY = Primary.class.getName();
 
     /**
      * @param beanType   The bean type
