@@ -5,7 +5,7 @@
  * you may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
  *
- * http://www.apache.org/licenses/LICENSE-2.0
+ * https://www.apache.org/licenses/LICENSE-2.0
  *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
@@ -15,10 +15,10 @@
  */
 package io.micronaut.inject.qualifiers;
 
+import io.micronaut.context.Qualifier;
 import io.micronaut.core.annotation.Internal;
 import io.micronaut.inject.BeanType;
 
-import javax.inject.Named;
 import java.lang.annotation.Annotation;
 import java.util.stream.Stream;
 
@@ -30,31 +30,33 @@ import java.util.stream.Stream;
  * @since 1.0
  */
 @Internal
-class AnnotationQualifier<T> extends NameQualifier<T> {
+class AnnotationQualifier<T> implements Qualifier<T> {
 
-    private final Annotation qualifier;
+    final Annotation annotation;
 
     /**
-     * @param qualifier The qualifier
+     * @param annotation The qualifier
      */
-    AnnotationQualifier(Annotation qualifier) {
-        super(qualifier.annotationType().getSimpleName());
-        this.qualifier = qualifier;
+    AnnotationQualifier(Annotation annotation) {
+        this.annotation = annotation;
     }
 
     @Override
     public <BT extends BeanType<T>> Stream<BT> reduce(Class<T> beanType, Stream<BT> candidates) {
-        String name;
-        if (qualifier instanceof Named) {
-            Named named = (Named) qualifier;
-            String v = named.value();
-            name = Character.toUpperCase(v.charAt(0)) + v.substring(1);
-
-        } else {
-            name = qualifier.annotationType().getSimpleName();
-        }
-
-        return reduceByAnnotation(beanType, candidates, name, qualifier.annotationType().getName());
+        String qualifiedName = annotation.annotationType().getName();
+        String annotationSimpleName = annotation.annotationType().getSimpleName();
+        return candidates.filter(candidate -> {
+            if (!QualifierUtils.matchType(beanType, candidate)) {
+                return false;
+            }
+            if (QualifierUtils.matchAny(beanType, candidate)) {
+                return true;
+            }
+            if (candidate.getAnnotationMetadata().hasDeclaredAnnotation(qualifiedName)) {
+                return true;
+            }
+            return QualifierUtils.matchByCandidateName(candidate, beanType, annotationSimpleName);
+        });
     }
 
     @Override
@@ -65,19 +67,16 @@ class AnnotationQualifier<T> extends NameQualifier<T> {
         if (o == null || getClass() != o.getClass()) {
             return false;
         }
-
-        AnnotationQualifier<?> that = (AnnotationQualifier<?>) o;
-
-        return qualifier.equals(that.qualifier);
+        return QualifierUtils.annotationQualifiersEquals(this, o);
     }
 
     @Override
     public int hashCode() {
-        return qualifier.hashCode();
+        return annotation.hashCode();
     }
 
     @Override
     public String toString() {
-        return '@' + qualifier.annotationType().getSimpleName();
+        return '@' + annotation.annotationType().getSimpleName();
     }
 }
