@@ -1,5 +1,5 @@
 /*
- * Copyright 2012-2018 the original author or authors.
+ * Copyright 2012-2020 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -45,13 +45,14 @@ import org.springframework.util.StringUtils;
  * @author Jon Brisbin
  * @author Dave Syer
  * @author Phillip Webb
+ * @since 1.0.0
  */
 public class Shell {
 
 	private static final Set<Class<?>> NON_FORKED_COMMANDS;
 
 	static {
-		Set<Class<?>> nonForked = new HashSet<Class<?>>();
+		Set<Class<?>> nonForked = new HashSet<>();
 		nonForked.add(VersionCommand.class);
 		NON_FORKED_COMMANDS = Collections.unmodifiableSet(nonForked);
 	}
@@ -86,9 +87,8 @@ public class Shell {
 	}
 
 	private Iterable<Command> getCommands() {
-		List<Command> commands = new ArrayList<Command>();
-		ServiceLoader<CommandFactory> factories = ServiceLoader.load(CommandFactory.class,
-				getClass().getClassLoader());
+		List<Command> commands = new ArrayList<>();
+		ServiceLoader<CommandFactory> factories = ServiceLoader.load(CommandFactory.class, getClass().getClassLoader());
 		for (CommandFactory factory : factories) {
 			for (Command command : factory.getCommands()) {
 				commands.add(convertToForkCommand(command));
@@ -113,18 +113,13 @@ public class Shell {
 		this.consoleReader.setHistoryEnabled(true);
 		this.consoleReader.setBellEnabled(false);
 		this.consoleReader.setExpandEvents(false);
-		this.consoleReader.addCompleter(new CommandCompleter(this.consoleReader,
-				this.argumentDelimiter, this.commandRunner));
+		this.consoleReader
+				.addCompleter(new CommandCompleter(this.consoleReader, this.argumentDelimiter, this.commandRunner));
 		this.consoleReader.setCompletionHandler(new CandidateListCompletionHandler());
 	}
 
 	private void attachSignalHandler() {
-		SignalUtils.attachSignalHandler(new Runnable() {
-			@Override
-			public void run() {
-				handleSigInt();
-			}
-		});
+		SignalUtils.attachSignalHandler(this::handleSigInt);
 	}
 
 	/**
@@ -147,8 +142,7 @@ public class Shell {
 		String version = getClass().getPackage().getImplementationVersion();
 		version = (version != null) ? " (v" + version + ")" : "";
 		System.out.println(ansi("Spring Boot", Code.BOLD).append(version, Code.FAINT));
-		System.out.println(ansi("Hit TAB to complete. Type 'help' and hit "
-				+ "RETURN for help, and 'exit' to quit."));
+		System.out.println(ansi("Hit TAB to complete. Type 'help' and hit RETURN for help, and 'exit' to quit."));
 	}
 
 	private void runInputLoop() throws Exception {
@@ -193,13 +187,13 @@ public class Shell {
 
 		private volatile Command lastCommand;
 
-		private final Map<String, String> aliases = new HashMap<String, String>();
+		private final Map<String, String> aliases = new HashMap<>();
 
 		ShellCommandRunner() {
 			super(null);
 		}
 
-		public void addAliases(String command, String... aliases) {
+		void addAliases(String command, String... aliases) {
 			for (String alias : aliases) {
 				this.aliases.put(alias, command);
 			}
@@ -225,9 +219,9 @@ public class Shell {
 		protected void afterRun(Command command) {
 		}
 
-		public boolean handleSigInt() {
+		boolean handleSigInt() {
 			Command command = this.lastCommand;
-			if (command != null && command instanceof RunProcessCommand) {
+			if (command instanceof RunProcessCommand) {
 				return ((RunProcessCommand) command).handleSigInt();
 			}
 			return false;
