@@ -1,5 +1,5 @@
 /*
- * Copyright 2012-2019 the original author or authors.
+ * Copyright 2012-2023 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -16,6 +16,9 @@
 
 package org.springframework.boot.web.servlet.support;
 
+import java.sql.Driver;
+import java.sql.DriverManager;
+import java.sql.SQLException;
 import java.util.Collections;
 
 import javax.servlet.Filter;
@@ -31,13 +34,15 @@ import org.springframework.boot.SpringApplication;
 import org.springframework.boot.builder.ParentContextApplicationContextInitializer;
 import org.springframework.boot.builder.SpringApplicationBuilder;
 import org.springframework.boot.context.event.ApplicationEnvironmentPreparedEvent;
+import org.springframework.boot.context.logging.LoggingApplicationListener;
 import org.springframework.boot.web.servlet.ServletContextInitializer;
 import org.springframework.boot.web.servlet.context.AnnotationConfigServletWebServerApplicationContext;
 import org.springframework.context.ApplicationContext;
 import org.springframework.context.ApplicationListener;
 import org.springframework.context.annotation.Configuration;
 import org.springframework.core.Ordered;
-import org.springframework.core.annotation.AnnotationUtils;
+import org.springframework.core.annotation.MergedAnnotations;
+import org.springframework.core.annotation.MergedAnnotations.SearchStrategy;
 import org.springframework.core.env.ConfigurableEnvironment;
 import org.springframework.util.Assert;
 import org.springframework.web.WebApplicationInitializer;
@@ -76,7 +81,7 @@ public abstract class SpringBootServletInitializer implements WebApplicationInit
 
 	/**
 	 * Set if the {@link ErrorPageFilter} should be registered. Set to {@code false} if
-	 * error page mappings should be handled via the server and not Spring Boot.
+	 * error page mappings should be handled through the server and not Spring Boot.
 	 * @param registerErrorPageFilter if the {@link ErrorPageFilter} should be registered.
 	 */
 	protected final void setRegisterErrorPageFilter(boolean registerErrorPageFilter) {
@@ -85,24 +90,42 @@ public abstract class SpringBootServletInitializer implements WebApplicationInit
 
 	@Override
 	public void onStartup(ServletContext servletContext) throws ServletException {
+		servletContext.setAttribute(LoggingApplicationListener.REGISTER_SHUTDOWN_HOOK_PROPERTY, false);
 		// Logger initialization is deferred in case an ordered
 		// LogServletContextInitializer is being used
 		this.logger = LogFactory.getLog(getClass());
-		WebApplicationContext rootAppContext = createRootApplicationContext(servletContext);
-		if (rootAppContext != null) {
-			servletContext.addListener(new ContextLoaderListener(rootAppContext) {
-				@Override
-				public void contextInitialized(ServletContextEvent event) {
-					// no-op because the application context is already initialized
-				}
-			});
+		WebApplicationContext rootApplicationContext = createRootApplicationContext(servletContext);
+		if (rootApplicationContext != null) {
+			servletContext.addListener(new SpringBootContextLoaderListener(rootApplicationContext, servletContext));
 		}
 		else {
-			this.logger.debug("No ContextLoaderListener registered, as " + "createRootApplicationContext() did not "
+			this.logger.debug("No ContextLoaderListener registered, as createRootApplicationContext() did not "
 					+ "return an application context");
 		}
 	}
 
+	/**
+	 * Deregisters the JDBC drivers that were registered by the application represented by
+	 * the given {@code servletContext}. The default implementation
+	 * {@link DriverManager#deregisterDriver(Driver) deregisters} every {@link Driver}
+	 * that was loaded by the {@link ServletContext#getClassLoader web application's class
+	 * loader}.
+	 * @param servletContext the web application's servlet context
+	 * @since 2.3.0
+	 */
+	protected void deregisterJdbcDrivers(ServletContext servletContext) {
+		for (Driver driver : Collections.list(DriverManager.getDrivers())) {
+			if (driver.getClass().getClassLoader() == servletContext.getClassLoader()) {
+				try {
+					DriverManager.deregisterDriver(driver);
+				}
+				catch (SQLException ex) {
+					// Continue
+				}
+			}
+		}
+	}
+
 	protected WebApplicationContext createRootApplicationContext(ServletContext servletContext) {
 		SpringApplicationBuilder builder = createSpringApplicationBuilder();
 		builder.main(getClass());
@@ -113,12 +136,12 @@ public abstract class SpringBootServletInitializer implements WebApplicationInit
 			builder.initializers(new ParentContextApplicationContextInitializer(parent));
 		}
 		builder.initializers(new ServletContextApplicationContextInitializer(servletContext));
-		builder.contextClass(AnnotationConfigServletWebServerApplicationContext.class);
+		builder.contextFactory((webApplicationType) -> new AnnotationConfigServletWebServerApplicationContext());
 		builder = configure(builder);
 		builder.listeners(new WebEnvironmentPropertySourceInitializer(servletContext));
 		SpringApplication application = builder.build();
 		if (application.getAllSources().isEmpty()
-				&& AnnotationUtils.findAnnotation(getClass(), Configuration.class) != null) {
+				&& MergedAnnotations.from(getClass(), SearchStrategy.TYPE_HIERARCHY).isPresent(Configuration.class)) {
 			application.addPrimarySources(Collections.singleton(getClass()));
 		}
 		Assert.state(!application.getAllSources().isEmpty(),
@@ -128,6 +151,7 @@ public abstract class SpringBootServletInitializer implements WebApplicationInit
 		if (this.registerErrorPageFilter) {
 			application.addPrimarySources(Collections.singleton(ErrorPageFilterConfiguration.class));
 		}
+		application.setRegisterShutdownHook(false);
 		return run(application);
 	}
 
@@ -172,6 +196,10 @@ public abstract class SpringBootServletInitializer implements WebApplicationInit
 		return builder;
 	}
 
+	/**
+	 * {@link ApplicationListener} to trigger
+	 * {@link ConfigurableWebEnvironment#initPropertySources(ServletContext, javax.servlet.ServletConfig)}.
+	 */
 	private static final class WebEnvironmentPropertySourceInitializer
 			implements ApplicationListener<ApplicationEnvironmentPreparedEvent>, Ordered {
 
@@ -196,4 +224,34 @@ public abstract class SpringBootServletInitializer implements WebApplicationInit
 
 	}
 
+	/**
+	 * {@link ContextLoaderListener} for the initialized context.
+	 */
+	private class SpringBootContextLoaderListener extends ContextLoaderListener {
+
+		private final ServletContext servletContext;
+
+		SpringBootContextLoaderListener(WebApplicationContext applicationContext, ServletContext servletContext) {
+			super(applicationContext);
+			this.servletContext = servletContext;
+		}
+
+		@Override
+		public void contextInitialized(ServletContextEvent event) {
+			// no-op because the application context is already initialized
+		}
+
+		@Override
+		public void contextDestroyed(ServletContextEvent event) {
+			try {
+				super.contextDestroyed(event);
+			}
+			finally {
+				// Use original context so that the classloader can be accessed
+				deregisterJdbcDrivers(this.servletContext);
+			}
+		}
+
+	}
+
 }
