@@ -1,5 +1,5 @@
 /*
- * Copyright 2002-2017 the original author or authors.
+ * Copyright 2002-2019 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -42,7 +42,11 @@ import org.springframework.util.StringUtils;
  * @author Ramnivas Laddad
  * @author Chris Beams
  * @since 2.5
+ * @deprecated As of Spring Framework 5.2, this class and related classes in this
+ * package have been replaced by {@link SimpleAnnotationMetadataReadingVisitor}
+ * and related classes for internal use within the framework.
  */
+@Deprecated
 class ClassMetadataReadingVisitor extends ClassVisitor implements ClassMetadata {
 
 	private String className = "";
@@ -117,6 +121,7 @@ class ClassMetadataReadingVisitor extends ClassVisitor implements ClassMetadata
 	}
 
 	@Override
+	@Nullable
 	public AnnotationVisitor visitAnnotation(String desc, boolean visible) {
 		// no-op
 		return new EmptyAnnotationVisitor();
@@ -165,11 +170,6 @@ class ClassMetadataReadingVisitor extends ClassVisitor implements ClassMetadata
 		return this.isAbstract;
 	}
 
-	@Override
-	public boolean isConcrete() {
-		return !(this.isInterface || this.isAbstract);
-	}
-
 	@Override
 	public boolean isFinal() {
 		return this.isFinal;
@@ -191,11 +191,6 @@ class ClassMetadataReadingVisitor extends ClassVisitor implements ClassMetadata
 		return this.enclosingClassName;
 	}
 
-	@Override
-	public boolean hasSuperClass() {
-		return (this.superClassName != null);
-	}
-
 	@Override
 	@Nullable
 	public String getSuperClassName() {
